---------------------------------------------------------------
--
--                    Top Graph'X
--
-- For any information call J.C. Mahieux :
-- Tel : (33) 1 69 26 97 88
-- Fax : (33) 1 69 26 97 89
--
-- Top Graph'X - 10 Allee de la Mare Jacob
--               91290 La Norville- FRANCE
--
---------------------------------------------------------------
--
------------------ Distribution and Copyright -----------------
-- Derivation   : MIT X Window System, V11.R4
--
-- This Ada language binding to the X Window System inherits
-- the MIT copyright. The following copyright must be included
-- in all software utilising this binding.
--
-- Copyright 1990, 1991 J.C. Mahieux and B. Maudry
-- Copyright 1985, 1986, 1987 by the Massachusetts Institute of Technology
--
with X_Types ;

package X_Cursors is
    use X_Types ;

    --====================================================================
    --  <Subprogram Name>
    --
    --  Create_Font_Cursor
    --
    --  <Description>
    --
    --  This function creates a cursor from a standard set of cursor
    --- shapes (Font cursors).
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --  Shape      : Specifies a cursor shape.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Alloc is raised if space could not be allocated.
    --  Bad_Match is raised if no matching cursor shape was found.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function Create_Font_Cursor
        ( Display_Id : in Display;
          Shape      : in Cursor_Shape) return Cursor;


    --====================================================================
    --  <Subprogram Name>
    --
    --  Create_Glyph_Cursor
    --
    --  <Description>
    --
    --  This function creates a cursor from font glyphs, that is, it will
    --  use an element (glyph) of a specified font as the cursor.
    --
    --  <Parameters>
    --
    --  Display_Id   :  The connection to the X Server.
    --  Source_Font  :  The font for the source glyph.
    --  Mask_Font    :  The font for the mask glyph.
    --  Source_Glyph :  A glyph character in the source font.
    --  Mask_Glyph   :  A glyph character in the mask font.
    --  Foreground   :  The RGB values for the foreground of Source.
    --  Background   :  The RGB values for the background of Source.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Alloc is raised if space could not be allocated.
    --  Bad_Match is raised if no matching cursor shape was found.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function Create_Glyph_Cursor
        ( Display_Id   : in Display;
          Source_Font  : in Font;
          Mask_Font    : in Font;
          Source_Glyph : in X_Character;
          Mask_Glyph   : in X_Character;
          Foreground   : in Color_Record;
          Background   : in Color_Record) return Cursor;


    --====================================================================
    --  <Subprogram Name>
    --
    --  Create_Pixmap_Cursor
    --
    --  <Description>
    --
    --  This function creates a cursor from two bitmaps.
    --
    --  <Parameters>
    --
    --  Display_Id :  The connection to the X Server.
    --  Source     :  The Cursor pixmap.
    --  Mask       :  The Mask pixmap.
    --  Foreground :  The RGB values for the foreground of Source.
    --  Background :  The RGB values for the background of Source.
    --  Hot_Spot   :  The coordinates of the hot spot.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Alloc is raised if space could not be allocated.
    --  Bad_Pixmap is raised if Source or Mask are invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function Create_Pixmap_Cursor ( Display_Id : in Display;
                                    Source     : in Pixmap;
                                    Mask       : in Pixmap;
                                    Foreground : in Color_Record;
                                    Background : in Color_Record;
                                    Hot_Spot   : in Point) return Cursor;


    --====================================================================
    --  <Subprogram Name>
    --
    --  Define_Cursor
    --
    --  <Description>
    --
    --  This procedure defines the cursor to be used in a window.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --  Window_Id  : Specifies the window.
    --  Cursor_Id  : Specifies the Id of the cursor.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Window is raised if the Window_Id is invalid.
    --  Bad_Cursor is raised if the Cursor_Id is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Define_Cursor ( Display_Id : in Display;
                              Window_Id  : in Window;
                              Cursor_Id  : in Cursor);


    --====================================================================
    --  <Subprogram Name>
    --
    --  Free_Cursor
    --
    --  <Description>
    --
    --  This procedure frees a cursor.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --  Cursor_Id  : Specfies the Id of the cursor.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Cursor is raised if Cursor_Id is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Free_Cursor ( Display_Id : in Display;
                            Cursor_Id  : in out Cursor);


    --======================================================================
    --  <Subprogram Name>
    --
    --  Query_Best_Cursor
    --
    --  <Description>
    --
    --  This procedure returns the best cursor width and height for the
    --  current display.
    --
    --  <Parameters>
    --
    --  Display_Id  : Specifies the connection to the X Server.
    --  Drawable_Id : Specifies the drawable.
    --  Width       : Specifies the requested width.
    --                Returns the closest width.
    --  Height      : Specifies the requested height.
    --                Returns the closest height.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Drawable is raised if the drawable is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    procedure Query_Best_Cursor ( Display_Id  : in Display;
                                  Drawable_Id : in Drawable;
                                  Width       : in out Pixels;
                                  Height      : in out Pixels);


    --====================================================================
    --  <Subprogram Name>
    --
    --  Recolor_Cursor
    --
    --  <Description>
    --
    --  This procedure changes the color of the specified cursor.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --  Cursor_Id  : Specifies the Id of the cursor.
    --  Foreground : Specifies the RGB values for the cursor foreground.
    --  Background : Specifies the RGB values for the cursor background.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Cursor is raised if Cursor_Id is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Recolor_Cursor ( Display_Id : in Display;
                               Cursor_Id  : in Cursor;
                               Foreground : in Color_Record;
                               Background : in Color_Record);

    --======================================================================
    --  <Subprogram Name>
    --
    --  Undefine_Cursor
    --
    --  <Description>
    --
    --  This procedure undefines the mouse cursor in the specified window.
    --  When the mouse is in the window, the parent's cursor will now then
    --  be used.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --  Window_Id  : Specifies the window.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Window is raised if the Window_Id is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    procedure Undefine_Cursor ( Display_Id : in Display;
                                Window_Id  : in Window);


end X_Cursors;
