---------------------------------------------------------------
--
--                    Top Graph'X
--
-- For any information call J.C. Mahieux :
-- Tel : (33) 1 69 26 97 88
-- Fax : (33) 1 69 26 97 89
--
-- Top Graph'X - 10 Allee de la Mare Jacob
--               91290 La Norville- FRANCE
--
---------------------------------------------------------------
--
------------------ Distribution and Copyright -----------------
-- Derivation   : MIT X Window System, V11.R4
--
-- This Ada language binding to the X Window System inherits
-- the MIT copyright. The following copyright must be included
-- in all software utilising this binding.
--
-- Copyright 1990, 1991 J.C. Mahieux and B. Maudry
-- Copyright 1985, 1986, 1987 by the Massachusetts Institute of Technology
--
with X_Types ;
with Text_Io ;
package X_Events is
    use X_Types ;

    --====================================================================
    --  <Subprogram Name>
    --
    --  Allow_Events
    --
    --  <Description>
    --
    --  This procedure is used to release some queued events if the client
    --  has caused a device to freeze. The function has no effect if the
    --  specified time is earlier than the last-grab time of the most
    --  recent active grab for that client, or if the specified time is
    --  later than the current X server time.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================
    procedure Allow_Events ( Display_Id : in Display;
                             Mode       : in Allow_Event_Mode;
                             Event_Time : in Time);

    --====================================================================
    --  <Subprogram Name>
    --
    --  Check_If_Event
    --
    --  <Description>
    --
    --  This procedure searches the event queue for the events satisfying
    --  with the specified predicate.
    --  Check_If_Event removes the next event in the queue that matches the
    --  predicate with the argument passed. It then copies the event
    --  into the supplied Event. Earlier events in the queue are not
    --  discarded and Event_Found is set to True. If the event you requested
    --  is not in the queue, Check_If_Event returns immediately and
    --  Event_Found is set to False.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================
    
    generic
        type Argument is private ;
        with function Predicate ( Display_Id : in Display;
                                  Item       : in Argument;
                                  The_Event  : in Event) return Boolean ;
        
    procedure Check_If_Event ( Display_Id  : in Display;
                               Item        : in Argument;
                               The_Event   : in out Event;
                               Event_Found : out Boolean );

    --======================================================================
    --  <Subprogram Name>
    --
    --  Check_Mask_Event
    --
    --  <Description>
    --
    --  This procedure searches the event queue for the events associated
    --  with the specified mask. Check_Mask_Event removes the next event in
    --  the queue that matches the event mask you passed. It then copies the
    --  event into the supplied Event and returns TRUE in Event_Found.
    --  Earlier events in the queue are not discarded.
    --  If the event you requested is not in the queue, Check_Mask_Event
    --  immediately returns and Event_Found is set to FALSE.
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    procedure Check_Mask_Event ( Display_Id  : in Display;
                                 Mask        : in Event_Mask_Type;
                                 The_Event   : in out Event;
                                 Event_Found : out Boolean);

    --======================================================================
    --  <Subprogram Name>
    --
    --  Check_Typed_Event
    --
    --  <Description>
    --
    --  This procedure searches the event queue for the events associated
    --  with the specified window.
    --  Check_Typed_Event removes the next event in the queue that
    --  matches the specified event type you passed. It then
    --  copies the event into the supplied Event and returns TRUE in
    --  Event_Found. Earlier events in the queue are not discarded. If the
    --  event you requested is not in the queue, Check_Typed_Event immediately
    --  returns and Event_Found is set to FALSE
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    procedure Check_Typed_Event ( Display_Id   : in Display;
                                  Event_Kind   : in Event_Type;
                                  The_Event    : in out Event;
                                  Event_Found  : out Boolean);

    --======================================================================
    --  <Subprogram Name>
    --
    --  Check_Typed_Window_Event
    --
    --  <Description>
    --
    --  This procedure searches the event queue for the events associated
    --  with the specified window.
    --  Check_Typed_Window_Event removes the next event in the queue that
    --  matches the window and specified event type you passed. It then
    --  copies the event into the supplied Event and returns TRUE in
    --  Event_Found. Earlier events in the queue are not discarded. If the
    --  event you requested is not in the queue, Check_Typed_Window_Event
    --  immediately returns and Event_Found is set to FALSE
    --
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    procedure Check_Typed_Window_Event ( Display_Id   : in Display;
                                         Window_Id    : in Window;
                                         Event_Kind   : in Event_Type;
                                         The_Event    : in out Event;
                                         Event_Found  : out Boolean);

    --======================================================================
    --  <Subprogram Name>
    --
    --  Check_Window_Event
    --
    --  <Description>
    --
    --  This procedure searches the event queue for the events associated with
    --  the specified window. Check_Window_Event removes the next event in the
    --  queue that matches both the window and the event mask supplied.
    --  It then copies the event's associated structure into the supplied
    --  Event and returns TRUE in Event_Found. If the event requested is
    --  not in the queue, Check_Window_Event immediately returns
    --  and Event_Found returns FALSE.
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    procedure Check_Window_Event ( Display_Id  : in Display;
                                   Window_Id   : in Window;
                                   Mask        : in Event_Mask_Type;
                                   The_Event   : in out Event;
                                   Event_Found : out Boolean);

    --====================================================================
    --  <Subprogram Name>
    --
    --  Events_Queued
    --
    --  <Description>
    --
    --  This function returns the number of events in the event queue.
    --  If the mode is Queued_Already, the number of events already in
    --  the queue is returned. If the mode is Queued_After_Flush, it
    --  flushes the output queue if necessary and returns the number of
    --  events. If the mode is Queued_After_Reading, it attempts to read
    --  the number of events without attempting a flush.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

      function Events_Queued
           ( Display_Id : in Display;
             Mode       : in Event_Queue_Mode_Type) return Natural;

    --====================================================================
    --  <Subprogram Name>
    --
    --  Flush
    --
    --  <Description>
    --
    --  This procedure flushes the output buffer.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Flush (Display_Id : in Display);

    --====================================================================
    --  <Subprogram Name>
    --
    --  Get_Motion_Events
    --
    --  <Description>
    --
    --  This function returns all events in the motion history buffer
    --  that fall between the specified start and stop times inclusive and
    --  that have coordinates that lie within (including borders) the
    --  specified window at its present placement. If the start time is
    --  later than the stop time or if the start time is in the future, no
    --  events are returned. Each time member in the Time_Coordinate list
    --  is set to the time in milliseconds. The x and y members are set to
    --  the coordinates of the pointer relative to the origin of the
    --  specified window.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Window
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function Get_Motion_Events ( Display_Id : in Display;
                                 Window_Id  : in Window;
                                 Start      : in Time;
                                 Stop       : in Time)
        return Time_Coord_List;

    --====================================================================
    --  <Subprogram Name>
    --
    --  If_Event
    --
    --  <Description>
    --
    --  This procedure searches the event queue for the events satisfying
    --  with the specified predicate.
    --  If_Event removes the next event in the queue that matches the
    --  predicate with the argument passed. It then copies the event
    --  into the supplied Event. Earlier events in the queue are not
    --  discarded. If the event you requested is not in the queue,
    --  If_Event blocks until one is received.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================
    
    generic
        type Argument is private ;
        with function Predicate ( Display_Id : in Display;
                                  Item       : in Argument;
                                  The_Event  : in Event) return Boolean ;
        
    procedure If_Event ( Display_Id : in Display;
                         Item       : in Argument;
                         The_Event  : in out Event);

    --====================================================================
    --  <Subprogram Name>
    --
    --  Lookup_Keysym
    --
    --  <Description>
    --
    --  This function uses a given keyboard event and the row specified
    --  to return the Key_Sym from the list that corresponds to the
    --  keycode in the event record. The Event must be of type Key_Release
    --  or Key_Pressed.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function Lookup_Keysym ( The_Event : in Event;
                             Row       : in Natural)
        return Key_Sym;

    --====================================================================
    --  <Subprogram Name>
    --
    --  Lookup_String
    --
    --  <Description>
    --
    --  This procedure is a convenience routine that can be used to map a
    --  key event to an ASCII string, using the modifier bits in the key
    --  event to deal with shift, lock and control. It returns the
    --  translated string into the user's buffer. It also detects any
    --  rebound Key_Syms and returns the specified bytes.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Lookup_String ( The_Event  : in Event;
                              Buffer     : in out String ;
                              Symbol     : out Key_Sym ;
                              Length     : out Natural ;
                              Status     : out Compose_Status_Record );

    --====================================================================
    --  <Subprogram Name>
    --
    --  Mask_Event
    --
    --  <Description>
    --
    --  This procedure flushes the output buffer then searches the event
    --  queue for the events associated with the specified mask.
    --  Mask_Event removes the next event in the queue that matches the
    --  event mask you passed. It then copies the event into the supplied
    --  Event. Earlier events in the queue are not discarded. If the event
    --  you requested is not in the queue, Mask_Event blocks until one is
    --  received.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

      procedure Mask_Event ( Display_Id : in Display;
                             Mask       : in Event_Mask_Type;
                             The_Event  : in out Event);

    --====================================================================
    --  <Subprogram Name>
    --
    --  Next_Event
    --
    --  <Description>
    --
    --  This procedure flushes the output buffer. If the event queue is
    --  empty, Next_Event blocks until an event is received. It then
    --  removes an event from the head of the queue and copies it
    --  into a supplied Event.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Next_Event ( Display_Id : in Display;
                           The_Event  : in out Event);


    --======================================================================
    --  <Subprogram Name>
    --
    --  Peek_Event
    --
    --  <Description>
    --
    --  This procedure flushes the output buffer. If the event queue is
    --  empty, Next_Event blocks until an event is received. It then
    --  retrieves (but does not remove) an event from the head of the queue
    --  and copies it into a supplied Event.
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    procedure Peek_Event ( Display_Id : in Display;
                           The_Event  : in out Event);

    --====================================================================
    --  <Subprogram Name>
    --
    --  Peek_If_Event
    --
    --  <Description>
    --
    --  This procedure searches the event queue for the events satisfying
    --  with the specified predicate.
    --  Peek_If_Event retrieves (but does not remove) the next event in the
    --  queue that matches the predicate with the argument passed. It then
    --  copies the event into the supplied Event. Earlier events in the queue
    --  are not discarded. If the event you requested is not in the queue,
    --  Peek_If_Event blocks until one is received.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================
    
    generic
        type Argument is private ;
        with function Predicate ( Display_Id : in Display;
                                  Item       : in Argument;
                                  The_Event  : in Event) return Boolean ;
        
    procedure Peek_If_Event ( Display_Id : in Display;
                              Item       : in Argument;
                              The_Event  : in out Event);

    --====================================================================
    --  <Subprogram Name>
    --
    --  Pending
    --
    --  <Description>
    --
    --  This function flushes the output buffer. It then returns the
    --  number of events received from the X Server but not yet removed
    --  from the event queue.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function Pending ( Display_Id : in Display) return Natural;

    --======================================================================
    --  <Subprogram Name>
    --
    --  Print_Error
    --
    --  <Description>
    --
    --  This procedure displays in text form the contents of an error event.
    --
    --  <Parameters>
    --
    --  File     : indicates where to print.
    --  An_Error : the error event to dump
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    procedure Print_Error ( File     : in Text_Io.File_Type;
                            An_Error : in Event ) ;
        
    --======================================================================
    --  <Subprogram Name>
    --
    --  Put_Back_Event
    --
    --  <Description>
    --
    --  This procedure pushes an event back onto the head of the current
    --  display's event queue. This can be useful if you have read an
    --  event and then decide that you would rather deal with it later.
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    procedure Put_Back_Event ( Display_Id : in Display;
                               The_Event  : in out Event);

    --====================================================================
    --  <Subprogram Name>
    --
    --  Queue_Length
    --
    --  <Description>
    --
    --  This function is used to return the length of the event queue for the
    --  connected display. Note that there may be more event that have not
    --  been read into the queue yet.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function Queue_Length (Display_Id : in Display) return Natural;


    --====================================================================
    --  <Subprogram Name>
    --
    --  Refresh_Keyboard_Mapping
    --
    --  <Description>
    --
    --  This procedure refreshes the stored modifier and keymap
    --  information. You usually call this when a Mapping_Notify event
    --  occurs. The result is to update a client application's knowledge
    --  of the keyboard.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Refresh_Keyboard_Mapping (The_Event : in Event);

    --====================================================================
    --  <Subprogram Name>
    --
    --  Select_Input
    --
    --  <Description>
    --
    --  This procedure requests that the Server report events associated
    --  with the event mask you pass to the Mask argument.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Select_Input ( Display_Id : in Display;
                             Window_Id  : in Window;
                             Mask       : in Event_Mask_Type);

    --====================================================================
    --  <Subprogram Name>
    --
    --  Send_Event
    --
    --  <Description>
    --
    --  This procedure identifies the destination window, determines
    --  which clients should receive the specified events, and ignores
    --  any active grabs.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Window and Bad_Value
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Send_Event ( Display_Id : in Display;
                           Window_Id  : in Window;
                           Propagate  : in Boolean;
                           Mask       : in Event_Mask_Type;
                           The_Event  : in Event);

    --====================================================================
    --  <Subprogram Name>
    --
    --  Sync
    --
    --  <Description>
    --
    --  This procedure flushes the output buffer then waits until all
    --  events (including error events) have been received and processed
    --  by the X Server.
    --  In addition, this procedure places the events recently processed
    --  by the X Server on the event queue.
    --
    --  If Discard is TRUE, Sync discards all events on the queue,
    --  including events that were on the queue before Sync was called.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Sync ( Display_Id : in Display;
                     Discard    : in Boolean := False);

    --======================================================================
    --  <Subprogram Name>
    --
    --  Window_Event
    --
    --  <Description>
    --
    --  This procedure searches the event queue for events associated with
    --  the specified window. Window_Event removes the next event in the queue
    --  that matches both the window and the event mask passed.
    --  It then copies the event into the supplied Event. Earlier events in
    --  the queue are not discarded.
    --
    --  If the event you requested is not in the queue, Window_Event blocks
    --  until one is received.
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    procedure Window_Event ( Display_Id : in Display;
                             Window_Id  : in Window;
                             Mask       : in Event_Mask_Type;
                             The_Event  : in out Event);

end X_Events;
