---------------------------------------------------------------
--
--                    Top Graph'X
--
-- For any information call J.C. Mahieux :
-- Tel : (33) 1 69 26 97 88
-- Fax : (33) 1 69 26 97 89
--
-- Top Graph'X - 10 Allee de la Mare Jacob
--               91290 La Norville- FRANCE
--
---------------------------------------------------------------
--
------------------ Distribution and Copyright -----------------
--
-- This package was completely designed by B. Maudry.
--
-- Copyright 1990, 1991 J.C. Mahieux and B. Maudry
--
with X_Types ;

package X_Window_Sets is
    use X_Types ;
    
    type Set is range 0 .. 256 ;
    subtype Real_Set is Set range 1 .. Set'last ;
    Null_Set : constant Set := 0 ;
    
    Max_Power         : constant := 15 ;
    Biggest_Id_In_Set : constant := 2 ** Max_Power - 1 ;
    type Id_In_Set is range 0 .. Biggest_Id_In_Set ;
    subtype Valid_Id_In_Set is Id_In_Set range 1 .. Biggest_Id_In_Set ;

    Null_Id_In_Set   : constant Id_In_Set := 0 ;
    
    subtype Size_Power is Id_In_Set range 1 .. Max_Power ;
    
    --===================================================================
    --  <Subprogram Name>
    --
    --  Initialise
    --
    --  <Description>
    --
    -- This procedure initialises the set management for the given display.
    -- It allows 2**Set_Size ids to be created in each set.
    -- It must be called only once.
    --
    --  <Parameters>
    --
    --  Display_Id   : Specifies the connection to the X Server.
    --  Set_Size     : Specifies the power of 2 to use as set size
    --
    --  <Exceptions Raised>
    --
    --  Program_Error is raised if it is called more than once.
    --
    --  <Called By>
    --
    --  External.
    --
    --===================================================================
        
    procedure Initialise ( Display_Id : in Display ;
                           Set_Size   : in Size_Power := 8 ) ;
    
    --===================================================================
    --  <Subprogram Name>
    --
    --  Set_Info
    --
    --  <Description>
    --
    -- This procedure returns the set and the id in set of the given window.
    -- If the window does not belong to any set, Null_Set is return and
    -- Id is set to Null_Id_In_Set.
    --
    --  <Parameters>
    --
    --  Display_Id   : Specifies the connection to the X Server.
    --  Window_Id    : Specifies the window.
    --  Set_Id       : Returns the set to which it belongs.
    --  Id           : Returns the id of the window in the set.
    --
    --  <Exceptions Raised>
    --
    --  None
    --
    --  <Called By>
    --
    --  External.
    --
    --===================================================================

    procedure Set_Info ( Display_Id : in Display ;
                         Window_Id  : in Window ;
                         Set_Id     : out Set ;
                         Id         : out Id_In_Set ) ;
                         
    --===================================================================
    --  <Subprogram Name>
    --
    --  Set_Of
    --
    --  <Description>
    --
    -- This function returns the set of the given window.
    -- Null_Set is returned if the window does not belong to any set.
    --
    --  <Parameters>
    --
    --  Display_Id   : Specifies the connection to the X Server.
    --  Window_Id    : Specifies the window.
    --
    --  <Exceptions Raised>
    --
    --  None
    --
    --  <Called By>
    --
    --  External.
    --
    --===================================================================

    function Set_Of ( Display_Id : in Display ;
                      Window_Id  : in Window ) return Set ;
                         
    --===================================================================
    --  <Subprogram Name>
    --
    --  Window_of
    --
    --  <Description>
    --
    -- This function returns the window of the given Id in the given set.
    --
    --  <Parameters>
    --
    --  Display_Id   : Specifies the connection to the X Server.
    --  Set_Id       : Specifies the set to which it belongs.
    --  Id           : Specifies the id of the window in the set.
    --
    --  <Exceptions Raised>
    --
    --  Constraint_Error is raised if the id is invalid
    --
    --  <Called By>
    --
    --  External.
    --
    --===================================================================

    function Window_Of ( Display_Id : in Display ;
                         Set_Id     : in Real_Set ;
                         Id         : in Valid_Id_In_Set ) return Window ;
                         
    generic
        type Index is (<>) ;
    package Creations_Within_Set is
        
        --===================================================================
        --  <Subprogram Name>
        --
        --  Initialise
        --
        --  <Description>
        --
        -- This procedure initialises the set managed the given display.
        -- It must be called only once before any call to the creations
        -- functions.
        --
        --  <Parameters>
        --
        --  Display_Id   : Specifies the connection to the X Server.
        --
        --  <Exceptions Raised>
        --
        --  Program_Error is raised if it is called more than once.
        --
        --  <Called By>
        --
        --  External.
        --
        --===================================================================

        procedure Initialise ( Display_Id : in Display ) ;
        
        --===================================================================
        --  <Subprogram Name>
        --
        --  Set_Managed
        --
        --  <Description>
        --
        -- This function returns the id of the set managed by this instance.
        -- It returns Null_Set if the instance is not initialised.
        --
        --  <Parameters>
        --
        --  None
        --
        --  <Exceptions Raised>
        --
        --  None
        --
        --  <Called By>
        --
        --  External.
        --
        --===================================================================
        
        function Set_Managed return Set ;
        
        --===================================================================
        --  <Subprogram Name>
        --
        --  Next_Id
        --
        --  <Description>
        --
        -- This function returns the next id to be allocated in the set.
        --
        --  <Parameters>
        --
        --  None.
        --
        --  <Exceptions Raised>
        --
        --  Constraint_Error is raised if no more Id is available.
        --
        --  <Called By>
        --
        --  External.
        --
        --===================================================================
        
        function Next_Id return Valid_Id_In_Set ;
        function Next_Id return Index ;
        
        --===================================================================
        --  <Subprogram Name>
        --
        --  Id_Of
        --
        --  <Description>
        --
        -- Two functions of this name are defined :
        -- The first returns the id in the set of the given window.
        -- Null_Id_In_Set is returned if the window does not belong to the set
        --
        -- The second return the user index of the given window.
        --
        --  <Parameters>
        --
        --  Window_Id    : Specifies the window.
        --
        --  <Exceptions Raised>
        --
        --  Constraint_Error is raised if the resulting index is invalid.
        --
        --  <Called By>
        --
        --  External.
        --
        --===================================================================
        
        function Id_Of ( Window_Id  : in Window ) return Id_In_Set ;
        function Id_Of ( Window_Id  : in Window ) return Index ;
        
        --===================================================================
        --  <Subprogram Name>
        --
        --  Window_Of
        --
        --  <Description>
        --
        -- This function returns the window of the given id in the set.
        --
        --  <Parameters>
        --
        --  Id    : Specifies the id to convert.
        --
        --  <Exceptions Raised>
        --
        --  Constraint_Error is raised if the id doesn't exist
        --
        --  <Called By>
        --
        --  External.
        --
        --===================================================================
        
        function Window_Of ( Id  : in Valid_Id_In_Set ) return Window ;
        function Window_Of ( Id  : in Index ) return Window ;
        
        --===================================================================
        --  <Subprogram Name>
        --
        --  Create_Window
        --
        --  <Description>
        --
        -- This function creates an unmapped subwindow for a specified parent
        -- window.
        --
        --  <Parameters>
        --
        --  Parent       : Specifies the parent window Id.
        --  Bounds       : Specifies the window dimensions.
        --  Border_Width : Specifies the border width.
        --  Depth        : Specifies the depth.
        --  Class        : Specifies the window's class.
        --  Visuals      : Specifies the visual type.
        --  Value_Mask   : Specifies which window attributes are to be set.
        --  Attributes   : Specifies the values of the window attributes.
        --
        --  <Exceptions Raised>
        --  
        --  Program_Error is raised on attempt to create
        --            more windows than allowed by the set size.
        --  If synchronisation is on, X_Error may be raised with the following
        --  reasons :
        --  Bad_Match is set if border width > 0 for Input_Only windows,
        --            or if the visual type and depth are unsupported for
        --            the screen.
        --  Bad_Alloc is set if space for the window could not be allocated
        --  Bad_Color is set if the Color map is invalid.
        --  Bad_Pixmap is set if a pixmap is invalid.
        --  Bad_Window is set if the parent is invalid.
        --
        --  <Called By>
        --
        --  External.
        --
        --===================================================================
        
        function Create_Window
            ( Parent       : in Window;
              Bounds       : in Rectangle;
              Border_Width : in Pixels;
              Depth        : in Depth_Type;
              Class        : in Window_Class;
              Visuals      : in Visual;
              Value_Mask   : in Wa_Mask_Type;
              Attributes   : in Set_Window_Attributes_Record ) return Window ;

        --===================================================================
        --  <Subprogram Name>
        --
        --  Create_Simple_Window
        --
        --  <Description>
        --
        --  This function creates an unmapped Input_Output subwindow for
        --  a specified parent window.
        --
        --  <Parameters>
        --
        --  Parent       : Specifies the parent window Id.
        --  Bounds       : Specifies the window dimensions.
        --  Border_Width : Specifies the border width.
        --  Border       : Specifies the border pixel value (color).
        --  Background   : Specifies the background pixel value.
        --
        --  <Exceptions Raised>
        --
        --  Program_Error is raised on attempt to create
        --            more windows than allowed by the set size.
        --  If synchronisation is on, X_Error may be raised with the following
        --  reasons :
        --  Bad_Alloc is set if space for the window could not be allocated
        --  Bad_Window is set if the parent is invalid.
        --
        --  <Called By>
        --
        --  External.
        --
        --===================================================================

        function Create_Simple_Window ( Parent       : in Window;
                                        Bounds       : in Rectangle;
                                        Border_Width : in Pixels;
                                        Border       : in Pixel_Value;
                                        Background   : in Pixel_Value )
            return Window ;
    end Creations_Within_Set ;
end X_Window_Sets ;
